@@ -43,6 +43,7 @@ namespace Sass {
     extern const char for_kwd[];
     extern const char from_kwd[];
     extern const char to_kwd[];
+    extern const char of_kwd[];
     extern const char through_kwd[];
     extern const char each_kwd[];
     extern const char in_kwd[];
@@ -161,6 +162,11 @@ namespace Sass {
     extern const char uri_chars[];
     extern const char real_uri_chars[];
 
+    // constants for selector combinators
+    extern const char selector_combinator_child[];
+    extern const char selector_combinator_general[];
+    extern const char selector_combinator_adjacent[];
+
     // some specific constant character classes
     // they must be static to be useable by lexer
     extern const char static_ops[];
