@@ -1,6 +1,7 @@
 require 'fileutils'
 require 'hrx'
 require 'pathname'
+require 'set'
 
 require_relative 'util'
 
@@ -25,12 +26,14 @@ class SassSpec::Directory
   # exists, this will load `subdir` from within `path/to/archive.hrx`.
   def initialize(path)
     @path = Pathname.new(path)
-    @path = @path.relative_path_from(Pathname.new(Dir.pwd)) if Pathname.new(path).absolute?
+    @path = @path.relative_path_from(Pathname.new(Dir.pwd)) if @path.absolute?
 
     # Always use forward slashes on Windows, because HRX requires them.
     @path = Pathname.new(@path.to_s.gsub(/\\/, '/')) if Gem.win_platform?
 
-    raise ArgumentError.new("#{@path} must be relative.") if @path.absolute?
+    if %w[.. .].include?(@path.each_filename.first)
+      raise ArgumentError.new("#{path} must be beneath the working directory")
+    end
 
     return if Dir.exist?(@path)
 
@@ -52,7 +55,7 @@ class SassSpec::Directory
       end
     end
 
-    raise "#{path} doesn't exist"
+    raise ArgumentError.new("#{path} doesn't exist")
   end
 
   # Returns the parent as a SassSpec::Directory, or `nil` if this is the root
@@ -62,7 +65,7 @@ class SassSpec::Directory
     dirname == "." ? nil : SassSpec::Directory.new(dirname)
   end
 
-  # Returns a list of all paths in this directory that match `pattern`, relative
+  # Returns a list of all files in this directory that match `pattern`, relative
   # to the directory root.
   #
   # This includes files within HRX files in this directory.
@@ -71,10 +74,26 @@ class SassSpec::Directory
       @archive.glob(pattern).select {|e| e.is_a?(HRX::File)}.map(&:path)
     else
       recursive = pattern.include?("**")
-      physical_pattern = recursive ? "{#{pattern},**/*.hrx}" : pattern
-      Dir.glob(File.join(@path, physical_pattern), File::FNM_EXTGLOB).flat_map do |path|
-        relative = Pathname.new(path).relative_path_from(@path).to_s
-        next relative unless recursive && path.end_with?(".hrx")
+      physical_pattern =
+        if recursive
+          "{#{File.join(@path, pattern)},#{File.join(@path, '**/*.hrx')}}"
+        else
+          File.join(@path, pattern)
+        end
+
+      seen = Set.new
+      Dir.glob(physical_pattern, File::FNM_EXTGLOB).flat_map do |path|
+        # Dir.glob() can emit the same path multiple times if multiple `{}`
+        # patterns cover it.
+        next [] if seen.include?(path)
+        seen << path
+
+        next [] if Dir.exists?(path)
+
+        absolute = Pathname.new(path).expand_path
+        relative = absolute.relative_path_from(@path.expand_path).to_s
+        next relative unless path.end_with?(".hrx")
+        next [] unless recursive
 
         dir = path[0...-".hrx".length]
         relative_dir = relative[0...-".hrx".length]
@@ -88,15 +107,27 @@ class SassSpec::Directory
   def file?(path)
     if hrx?
       @archive[path].is_a?(HRX::File)
+    elsif (dir, basename = split_if_nested(path))
+      dir.file?(basename)
     else
-      File.exist?(File.join(@path, path))
+      File.file?(File.join(@path, path))
     end
+  rescue ArgumentError, HRX::Error
+    # If we get a directory-doesn't-exist error for a nested directory, return
+    # false. This could catch unrelated errors, but it's probably not likely
+    # enough to be worth creating a custom exception class.
+    return false
   end
 
   # Reads the file at `path` within this directory.
   def read(path)
-    return @archive.read(path) if hrx?
-    File.read(File.join(@path, path), binmode: true, encoding: "ASCII-8BIT")
+    if hrx?
+      @archive.read(path)
+    elsif (dir, basename = split_if_nested(path))
+      dir.read(basename)
+    else
+      File.read(File.join(@path, path), binmode: true, encoding: "ASCII-8BIT")
+    end
   end
 
   # Writes `contents` to `path` within this directory.
@@ -104,6 +135,8 @@ class SassSpec::Directory
     if hrx?
       @archive.write(path, contents, comment: :copy)
       _write!
+    elsif (dir, basename = split_if_nested(path))
+      dir.write(basename, contents)
     else
       File.write(File.join(@path, path), contents, binmode: true)
     end
@@ -117,6 +150,8 @@ class SassSpec::Directory
     if hrx?
       @archive.delete(path)
       _write!
+    elsif (dir, basename = split_if_nested(path))
+      dir.delete(basename, if_exists: if_exists)
     else
       File.unlink(File.join(@path, path))
     end
@@ -124,17 +159,23 @@ class SassSpec::Directory
 
   # Renames the file at `old` to `new`.
   def rename(old, new)
-    if hrx?
-      unless old_file = @archive[old]
+    old_dir, old_basename = split_if_nested(old) || [self, old]
+    new_dir, new_basename = split_if_nested(new) || [self, new]
+
+    if old_dir.hrx? && new_dir.hrx?
+      unless old_file = old_dir.archive[old_basename]
         raise "#@path/old doesn't exist"
       end
 
-      @archive.add(HRX::File.new(new, old_file.contents, comment: old_file.comment),
-                   after: old_file)
-      @archive.delete(old)
-      _write!
+      new_dir.archive.add(
+        HRX::File.new(new_basename, old_file.content, comment: old_file.comment),
+        after: new_dir == old_dir ? old_file.path : nil)
+      new_dir._write!
+
+      old_dir.delete(old_basename)
     else
-      File.rename(File.join(@path, old), File.join(@path, new))
+      new_dir.write(new_basename, old_dir.read(old_basename))
+      old_dir.delete(old_basename)
     end
   end
 
@@ -145,11 +186,7 @@ class SassSpec::Directory
         _delete_parent!
       else
         @parent_archive.delete(@path_in_parent, recursive: true)
-        if @parent_archive.entries.empty?
-          _delete_parent!
-        else
-          _write!
-        end
+        _write!
       end
     else
       FileUtils.rm_rf(@path)
@@ -165,7 +202,7 @@ class SassSpec::Directory
     return yield unless @archive
 
     files = @archive.entries.select {|entry| entry.is_a?(HRX::File)}.to_a
-    if parent.hrx? && files.any? {|file| _reaches_out?(file)}
+    if parent && parent.hrx? && files.any? {|file| _reaches_out?(file)}
       return parent.with_real_files {yield}
     end
 
@@ -198,8 +235,29 @@ class SassSpec::Directory
     @path.to_s
   end
 
+  protected
+
+  # The directory's underlying HRX archive. `nil` if `hrx?` is `false`.
+  attr_reader :archive
+
+  # Writes `@parent_archive` to disk.
+  def _write!
+    if @parent_archive.entries.empty?
+      _delete_parent!
+    else
+      @parent_archive.write!(@parent_archive_path)
+    end
+  end
+
   private
 
+  # If `path` points to a subdirectory of this directory, returns the nested
+  # `Directory` object and the basename of the file. Otherwise, returns `nil`.
+  def split_if_nested(path)
+    dirname, basename = File.split(path)
+    dirname == '.' ? nil : [SassSpec::Directory.new(@path.join(dirname)), basename]
+  end
+
   # Returns whether `file` contains enough `../` references to reach outside
   # this directory.
   def _reaches_out?(file)
@@ -207,11 +265,6 @@ class SassSpec::Directory
     file.content.scan(%r{(?:\.\./)+}).any? {|match| match.count("/") > depth}
   end
 
-  # Writes `@parent_archive` to disk.
-  def _write!
-    @parent_archive.write!(@parent_archive_path)
-  end
-
   # Deletes `@parent_archive` from disk and from the archive cache.
   def _delete_parent!
     File.unlink(@parent_archive_path)
